@@ -40,53 +40,62 @@ sub new {
 }
 
 sub _get {
-	my ($self, $name, $args, $val) = @_;
-	my $res = $self->{_stash}->{$name};
-	return unless defined($res);
-	$res = $args->{inflate}->($res) if $args->{inflate};
-	return $res;
+	my ($self, $name, $val) = @_;
+	return $self->{_stash}->{$name};
 }
 
 sub _delete_children {
-	my ($self, $args) = @_;
-	for my $d (@{ $args->{children} || [] }) {
+	my ($self, $name) = @_;
+	for my $d (@{ $self->_get_args($name)->{children} || [] }) {
 		my $f = "delete_$d";
 		$self->$f;
 	}
 }
 
 sub _set {
-	my ($self, $name, $args, $val) = @_;
-	$self->{_stash}->{$name} = $args->{deflate} ? $args->{deflate}->($val) : $val;
-	$self->_delete_children($args);
+	my ($self, $name, $val) = @_;
+	$self->{_stash}->{$name} = $val;
+	$self->_delete_children($name);
 }
 
 sub _delete {
-	my ($self, $name, $args) = @_;
+	my ($self, $name) = @_;
 	my $res = delete $self->{_stash}->{$name};
-	$self->_delete_children($args);
+	$self->_delete_children($name);
 	return $res;
 }
 
+sub add_class_dbi_var {
+	my ($class, $var, $dbi_class) = @_;
+	$class->add_var($var, inflate => sub {
+		return $dbi_class->retrieve(shift());
+	}, deflate => sub { return shift()->id });
+}
+
+sub _get_args {
+	my ($self, $name) = @_;
+	my $p_args = "$name\_args";
+	return $self->$p_args;
+}
+
 sub add_var {
 	my ($class, $name, %args) = @_;
 	no strict 'refs';
 	*{ "$class\::get_$name" } = sub {
-		return shift()->_get($name, \%args, @_);
+		return shift()->_get($name, @_);
 	};
 	*{ "$class\::set_$name" } = sub {
-		return shift()->_set($name, \%args, @_);
+		return shift()->_set($name, @_);
 	};
 	*{ "$class\::delete_$name" } = sub {
-		return shift()->_delete($name, \%args, @_);
+		return shift()->_delete($name, @_);
 	};
 	*{ "$class\::$name\_args" } = sub { return \%args; };
 
 	$args{children} = [] unless $args{children};
 
 	for my $d (@{ $args{depends_on} || [] }) {
-		my $p_args = "$d\_args";
-		push @{ $class->$p_args->{children} }, $name;
+		push @{ $class->_get_args($d)->{children} }, $name;
 	}
 }
 
@@ -95,13 +104,24 @@ sub session_id { return shift()->{session_id}; }
 sub write_stash {
 	my $self = shift;
 	my $file = $self->sessions_dir . "/" . $self->session_id;
-	lock_store($self->{_stash}, $file);
+	my %s;
+	while (my ($n, $v) = each %{ $self->{_stash} }) {
+		my $in = $self->_get_args($n)->{deflate};
+		$s{$n} = $in ? $in->($v) : $v;
+	}
+	lock_store(\%s, $file);
 }
 
 sub read_stash {
 	my $self = shift;
 	my $file = $self->sessions_dir . "/" . $self->session_id;
-	$self->{_stash} = -f $file ? lock_retrieve($file) : {};
+	my $s = -f $file ? lock_retrieve($file) : {};
+	my %stash;
+	while (my ($n, $v) = each %$s) {
+		my $d = $self->_get_args($n)->{inflate};
+		$stash{$n} = $d ? $d->($v) : $v;
+	}
+	$self->{_stash} = \%stash;
 }
 
 1;
